/*
 * Created on Jul 6, 2005
 *
 */
package tacle.core;

import java.util.Set;
import org.eclipse.jdt.core.IMethod;

/**
 * Generic interface to a call graph construction analysis.
 */

public interface ICallGraphAnalysis {
    
    /** 
     * Run the analysis, starting from the given main method.
     */
    void run(IMethod mainMethod);
    
    /**
     * Returns a set of <code>IMethod</code> references, representing
     * the methods reachable from the main method. Not guarantted to
     * be complete, due to the limitations of <code>IType</code> and
     * <code>IMethod</code>. Static initializers are not represented 
     * by IMethods, and any calls from them are not represented.
     * 
     * The current version does not deal with reflective calls and throw 
     * clauses. Methods declared in anonymous classes are not reported.
     * The body of a method declared in an anonymous class created that
     * is declared in a method may be wrongly reported as belonging
     * to the enclosing method. See 
     * http://presto.cse.ohio-state.edu/tacle/index.html
     * for more details on the current implementation.   
     * 
     * Current version works only with features of Java 1.4.
     */
    Set getReachableIMethods();

    /**
     * Returns a set of <code>IType</code> references, representing
     * the class and interface types that are initialized in code
     * reachable from the main method. Not guarantted to be complete,
     * due to the limitations of <code>IType</code> and
     * <code>IMethod</code>. 
     * 
     * Current version does not report anonymous types. See 
     * http://presto.cse.ohio-state.edu/tacle/index.html
     * for more details on the current implementation. 
     */
    Set getInitializedITypes();

    /** 
     * Returns a set of <code>IMethod</code> references for all
     * methods called by the given method. The given method should be
     * reachable. Not guarantted to be complete, due to the
     * limitations of <code>IType</code> and
     * <code>IMethod</code>.
     *  
     * The current version does not deal with reflective calls and throw 
     * clauses. Methods declared in anonymous classes are not reported.
     * The body of a method declared in an anonymous class created that
     * is declared in a method may be wrongly reported as belonging
     * to the enclosing method. See 
     * http://presto.cse.ohio-state.edu/tacle/index.html
     * for more details on the current implementation.   
     * 
     * Current version works only with features of Java 1.4.
     */    
    Set getCalledIMethods(IMethod method);

    /** 
     * Returns the set of call sites in the method. The given method
     * should be reachable. 
     * @see ICallSite
     */    
    Set getCallSites(IMethod method);

    /** 
     * Reports true the given method was analyzed from source code, and false
     * otherwise. The given method should be reachable.
     */
    boolean analyzedFromSourceCode(IMethod method);

    /** 
     * Looks for a call site in the given method such that the start
     * position and the length of the source range match exactly the
     * given values. This method can be used to obtain information
     * about an AST node representing a call site: 
     * <blockquote> <pre>
     * IMethod m = ...;
     * MethodInvocation n = ...; // node in m's AST
     * CallGraphAnalysis c = ...;
     * CallSite s = c.getCallSite(m, n.getStartPosition(), n.getLength());
     * if (s != null) // null will be returned if there is no match
     *    System.out.println("Num callees: " + s.getCalledIMethods().size());
     * </pre> </blockquote>
     * The given method should be reachable. 
     */
    ICallSite getCallSite(IMethod method, int pos, int length);

    /** 
     * Returns a set of <code>IMethod</code> references for all
     * methods calling the given method. The given method should be
     * reachable. Not guarantted to be complete, due to the
     * limitations of <code>IType</code> and
     * <code>IMethod</code>.
     *  
     * The current version does not deal with reflective calls and throw 
     * clauses. Methods declared in anonymous classes are not reported.
     * The body of a method declared in an anonymous class created that
     * is declared in a method may be wrongly reported as belonging
     * to the enclosing method. See 
     * http://presto.cse.ohio-state.edu/tacle/index.html
     * for more details on the current implementation.   
     * 
     * Current version works only with features of Java 1.4.
     */
    public Set getCallingMethods(IMethod method);

}
    
