/*
 * Created on Jul 7, 2005
 *
 */
package tacle.examples.dotgraph;

import org.eclipse.jdt.core.IMethod;
import java.io.*;
import tacle.core.*;
import java.util.*;

public class DotGraphGenerator {
	
	//flag that specifies the type of graph to generate
	//true = multigraph with an arbitrary number assigned to each callsite
	//	the edges are labled with the number
	//false = call graph with no information about the callsites DEFAULT
	private boolean detailedGraph = true;
	
	//analysis interface
	private ICallGraphAnalysis cga = null;
	
	//Simple function that opens a BufferedWriter to
	//the dot file and returns it, returns null and prints
	//error if it is unable to open the file
	private BufferedWriter openDotFile (String path){
		BufferedWriter out=null;
		
		try{
			out = new BufferedWriter(new FileWriter(path,false));
			out.write("digraph \"\" {");
			out.newLine();
		}
		catch(IOException e){
			System.out.println("Unable to open file "+path);
			e.printStackTrace();
		}
		
		return out;
	}
	
	//Closes the dot file
	private void closeDotFile(BufferedWriter out){
			
		try{
			out.write("}\n");
			out.close();
		}
		catch(IOException e){
			System.out.println("Unable to close dot file ");
			e.printStackTrace();
		}
		
	}
	
	//Function that writes to the dot file and does the 
	//appropriate Graphviz format 
	private void writeDotFile(BufferedWriter out,IMethod current, IMethod next, 
			String label){
		
		try{
			
			out.write(current.getElementName()+" -> "+next.getElementName()
					+"[label=\""+label+"\"]");
			out.newLine();
		}
		catch(IOException e){
			System.out.println("Unable write to dot file ");
			e.printStackTrace();
		}
		
	}
	
	
	//this method is the the only public method for this class
	//the parameters: 
	//IMethod mainMethod = the IMethod for the main method being analyzed
	//String outFilePath = full path to the desired dot file
	public boolean run(IMethod mainMethod, String outFilePath){
		//create the dot file do simple check to make sure it was created
		BufferedWriter out = openDotFile(outFilePath);
		if (out == null){
			return false;
		}
		//create and analysis factory
		AnalysisFactory anaFac =  new AnalysisFactory();
		//get the RTA implementation of ICallGraphAnalysis
		cga = anaFac.getAnalysis(AnalysisFactory.RTA);
		
		//use flag to determine which type of graph to generate
		try{
			if(detailedGraph){
				runDetailed(mainMethod,out);
				}else {
				runSimple(mainMethod,out);	
			}
		}finally{
			closeDotFile(out);
			anaFac = null;
			cga = null;
		}
		return true;
	}
	
	//create a multigraph call graph with the callsite info as edge labels
	//IMethod mainMethod = is the IMethod for the main method being analyzed
	//BufferedWriter out = the opened BufferedWriter to the dot file that will contain
	//	the call graph
	private void runDetailed(IMethod mainMethod, BufferedWriter out){
		//necessary if there is a loop in the call graph 
		Set alreadyAnalyzed = new HashSet();
		//run the analysis on the passed in main method
		cga.run(mainMethod);
		List workList = new ArrayList();
		
		//add main to the work list
		workList.add(mainMethod);
		//arbitrary number for the method to be used in the edge label
		int mCount = 1;
		
		while(!workList.isEmpty()){
			//remove any method from the worklist
			IMethod current = (IMethod)workList.remove(0);
			//get the set of callsites in the current method
			Set callS = cga.getCallSites(current);
			//arbitrary number of the callsites
			int csCount = 1;
			if(callS != null){
				//iterate over all the callsites in the method
				for(Iterator i = callS.iterator(); i.hasNext();){
					ICallSite cs = (ICallSite)i.next();
					//Get the set of possible target methods for the current callsite
					Set csMethods = cs.getCalledIMethods();
					if(csMethods != null){
						//iterator over the possible target methods
						for(Iterator iM = csMethods.iterator(); iM.hasNext();){
							IMethod next = (IMethod)iM.next();	
							//Only add user methods to the worklist that have not 
							//already been analyzed
							if(cga.analyzedFromSourceCode(next) && !alreadyAnalyzed.contains(next)){
								workList.add(next);
								alreadyAnalyzed.add(next);
								
							}
							//only add an edge if the target method is user defined
							if(cga.analyzedFromSourceCode(next)){
								writeDotFile(out,current,next,mCount+"."+csCount);
							}
						}
					}
					csCount++;
				}
			}
			mCount++;
		}
	}
	
	
	//Create a simple call graph 
	//IMethod mainMethod = is the IMethod for the main method being analyzed
	//BufferedWriter out = the opened BufferedWriter to the dot file that will contain
	//	the call graph
	private void runSimple(IMethod mainMethod, BufferedWriter out){
		//necessary if there is a loop in the call graph 
		Set alreadyAnalyzed = new HashSet();
		
		//run the analysis
		cga.run(mainMethod);
		List workList = new ArrayList();
		//add the mainMethod to the work list
		workList.add(mainMethod);
		
		while(!workList.isEmpty()){
			//remove any method from the work list
			IMethod current = (IMethod)workList.remove(0);
			//get the methods called from the current method
			//this will return both library and user methods or null if it is empty
			Set callM = cga.getCalledIMethods(current);
		
			if(callM != null){
				//iterate over all the methods called in the current method
				for(Iterator i = callM.iterator(); i.hasNext();){
					IMethod next = (IMethod)i.next();
					//Only add edged for user code and method that are not 
					//already anaylyzed 
					if(cga.analyzedFromSourceCode(next)&& !alreadyAnalyzed.contains(next)){
						//add new methods to the worklist
						workList.add(next);
						alreadyAnalyzed.add(next);
						//add the new edge to the call graph
						writeDotFile(out,current,next,"");
					}
				}
			}
		}
		
	
	}

}
