/*
 * Created on May 22, 2005
 *
 */
package tacle.internal.rta;

import java.util.Iterator;
import java.util.Set;

import org.eclipse.jdt.core.dom.ClassInstanceCreation;
import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.Initializer;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.StringLiteral;
import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
import org.eclipse.jdt.core.dom.SuperMethodInvocation;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.TypeDeclaration;

import tacle.internal.rta.data.ASTMethod;
import tacle.internal.rta.data.IWorklistMethod;
import tacle.internal.rta.data.InfoDictionary;
import tacle.internal.rta.data.IWorklistType;

//import tacle.internal.rta.data.ASTClass;

/**
 * @author Mariana L Barca
 * 
 * to be used for the whole compilation unit of the class that declares main
 */

public class MainMethodVisitor extends MethodVisitor {

	// true if visiting the main method or the static init or the class
	private boolean isInMainOrInit;

	// the class for which we analyze main & the static initializer
	private IWorklistType cls;

	// field relevant only after traversal
	private boolean hasInitializer;

	private boolean isInsideClass;

	public boolean hasStaticInitializer() {
		return hasInitializer;
	}

	// constructor
	MainMethodVisitor(RTAMethodWorklist worklist, IWorklistType cls) {
		super(worklist, null);
		isInMainOrInit = false;
		hasInitializer = false;
		isInsideClass = false;
		this.cls = cls;

		// create a dummy main method
		this.method = new ASTMethod("", "main", null, null, null);
	}

	//////////// visit methods /////////////

	public boolean visit(ClassInstanceCreation node) {
		if (isInMainOrInit) {
			//System.out.println("Visiting... " + node);
			super.visit(node);
		}
		return true;
	}

	public boolean visit(ConstructorInvocation node) {
		if (isInMainOrInit) {
			//System.out.println("Visiting... " + node);
			super.visit(node);
		}
		return true;
	}

	// visit only static field declarations
	public boolean visit(FieldDeclaration node) {
		if (isInsideClass && Modifier.isStatic(node.getModifiers())) {
			isInMainOrInit = true;
			hasInitializer = true;
			//System.out.println(node + " in " + cls);
		} else {
			isInMainOrInit = false;
		}
		return isInMainOrInit;
	}

	// flags say if this is a static block
	public boolean visit(Initializer node) {
		if (isInsideClass && Modifier.isStatic(node.getModifiers())) {
			isInMainOrInit = true;
			hasInitializer = true;
			//System.out.println(node + " in " + cls);
		} else {
			isInMainOrInit = false;
		}
		return isInMainOrInit;
	}

	public boolean visit(MethodDeclaration node) {
		if (isInsideClass
				&& node.getName().getFullyQualifiedName().equals("main")) {
			//System.out.println("Visiting... " + node);
			IMethodBinding mb = node.resolveBinding();
			ASTMethod m = InfoDictionary.v().getASTMethod(mb.getKey());
			if (m == null) {
				IWorklistType[] params = new IWorklistType[mb
						.getParameterTypes().length];
				for (int j = 0; j < params.length; j++) {
					params[j] = InfoDictionary.v().getTypeForBinding(
							mb.getParameterTypes()[j]);
				}
				IWorklistType declClass = InfoDictionary.v().getTypeForBinding(
						mb.getDeclaringClass());
				IWorklistType retType = InfoDictionary.v().getTypeForBinding(
						mb.getReturnType());
				m = new ASTMethod(mb.getKey(), mb.getName(), declClass,
						retType, params);
				InfoDictionary.v().setASTMethod(mb.getKey(), m);
			}
			if (m != null) {
				// we have to replace the fake main method
				ASTMethod fakeMain = (ASTMethod)this.method;
				this.method = m;
				
				// transfer the resolved call sites
				Set resolvedSites = fakeMain.getResolvedCallSites();
				for(Iterator it = resolvedSites.iterator(); it.hasNext(); ) {
					this.method.addResolvedCallSite((ResolvedCallSite)it.next());
				}
			}
			worklist.addMethodAfterAnalysis(m);
			isInMainOrInit = true;
		} else {
			isInMainOrInit = false;
		}
		return isInMainOrInit;
	}

	public boolean visit(MethodInvocation node) {
		if (isInMainOrInit) {
			//System.out.println("Visiting... " + node);
			super.visit(node);
		}
		return true;
	}

	// this can be the name of a static field
	public boolean visit(SimpleName node) {
		if (isInMainOrInit) {
			super.visit(node);
		}
		return true;
	}

	public boolean visit(SuperConstructorInvocation node) {
		return false;
	}

	public boolean visit(SuperMethodInvocation node) {
		return false;
	}

	// this is to remember the parent
	// in case super was not called explicitly and
	// we have to call it manually
	public boolean visit(TypeDeclaration node) {
		ITypeBinding itb = node.resolveBinding();
		IWorklistType temp = InfoDictionary.v().getTypeForBinding(itb);
		if (cls.equals(temp)) {
			isInsideClass = true;
		} else if (isInsideClass) {
			return false;
		}
		// go down other types because the type we are searching for
		// may be inner
		return true;
	}

	// set isInClass to false after desired class is visited
	public void endVisit(TypeDeclaration node) {
		ITypeBinding itb = node.resolveBinding();
		IWorklistType temp = InfoDictionary.v().getTypeForBinding(itb);
		if (cls.equals(temp)) {
			isInsideClass = false;
		}
	}

}
