/*
 * Created on Jun 17, 2005
 *
 */
package tacle.internal.rta.data;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import tacle.internal.rta.ResolvedCallSite;

/**
 * @author Mariana L Barca
 *  
 */
public class MethodInfo implements IWorklistMethod {
	// method info
	private int methodId;

	private String name;

	// return type
	private int returnTypeId;

	private int returnPackageId;

	private IWorklistType returnType;

	// params
	private int[][] paramTypeIds; // package, type

	private IWorklistType[] paramTypes; // can be ClassInfo or String

	// declaring class
	private int declaringClassId;

	private int declaringPackageId;

	private ClassInfo declaringClass;

	// abstract/ non abstract

	private int modifiers;

	// fields containing RTA info
	private List instantiatedClassIds; // Member: Integer in pairs, pkg id, cls

	private Set instantiatedClasses; // Member: ClassInfo

	private List initializedClassIds; // Member: Integer in pairs, pkg id, cls

	private Set initializedClasses; // Member: ClassInfo

	private Set callSites; // Member: CallSiteInfo

	private Set resolvedCallSites; // Member: ResolvedCallSite
	
	// constructors
	public MethodInfo() {
		initializedClassIds = new ArrayList();
		instantiatedClassIds = new ArrayList();
		callSites = new HashSet();
		resolvedCallSites = new HashSet();
	}

	// helper methods that populate members
	private void populateReturnType() {
		if (returnPackageId > 0) {
			returnType = InfoDictionary.v().getClassInfo(returnTypeId,
					returnPackageId);
		} else { // non class type
			returnType = InfoDictionary.v().getOtherType(returnTypeId);
		}
	}

	private void populateParamTypes() {
		paramTypes = new IWorklistType[paramTypeIds.length];
		for (int i = 0; i < paramTypes.length; i++) {
			if (paramTypeIds[i][0] > 0) {
				paramTypes[i] = InfoDictionary.v().getClassInfo(
						paramTypeIds[i][1], paramTypeIds[i][0]);
			} else {
				paramTypes[i] = InfoDictionary.v().getOtherType(
						paramTypeIds[i][1]);
			}
		}
	}

	private void populateDeclaringClass() {
		declaringClass = InfoDictionary.v().getClassInfo(declaringClassId,
				declaringPackageId);
	}

	private void populateInitializedClasses() {
		initializedClasses = new HashSet();
		for (Iterator it = initializedClassIds.iterator(); it.hasNext();) {
			Integer pkgId = (Integer) it.next();
			if (!it.hasNext()) {
				// error
				throw new RuntimeException(
						"initialized classes ids not in pairs!");
			}
			Integer clsId = (Integer) it.next();
			initializedClasses.add(InfoDictionary.v().getClassInfo(
					clsId.intValue(), pkgId.intValue()));
		}
	}

	private void populateInstantiatedClasses() {
		instantiatedClasses = new HashSet();
		for (Iterator it = instantiatedClassIds.iterator(); it.hasNext();) {
			Integer pkgId = (Integer) it.next();
			if (!it.hasNext()) {
				// error
				throw new RuntimeException(
						"instantiated classes ids not in pairs!");
			}
			Integer clsId = (Integer) it.next();
			instantiatedClasses.add(InfoDictionary.v().getClassInfo(
					clsId.intValue(), pkgId.intValue()));
		}
	}

	// IWorklistMethod interface methods
	public Set getResolvedCallSites() {
		return resolvedCallSites;
	}
	
	public IWorklistType getDeclaringClass() {
		if (declaringClass == null) {
			populateDeclaringClass();
		}
		return declaringClass;
	}

	public String getKey() {
		// keys for constructors look somehow strange...
		if (declaringClass == null) {
			populateDeclaringClass();
		}
		if (returnType == null) {
			populateReturnType();
		}
		if (paramTypes == null) {
			populateParamTypes();
		}
		String result = declaringClass.getCompleteName() + "/" + returnType
				+ name + "(";
		for (int i = 0; i < paramTypes.length; i++) {
			result += (i > 0 ? "," : "") + paramTypes[i].getCompleteName();
		}
		return result + ")";
	}

	public IWorklistType getReturnType() {
		if (returnTypeId == 0) {
			throw new RuntimeException("WEIRD - type with id 0"); // this looks
			// like an
			// error
		} else if (returnType == null) {
			populateReturnType();
		}
		return returnType;
	}

	public IWorklistType[] getParameters() {
		if (paramTypes == null) {
			populateParamTypes();
		}
		return paramTypes;
	}

	// getters & setters
	public void addResolvedCallSite(ResolvedCallSite rcs) {
		resolvedCallSites.add(rcs);
	}
	
	public int getDeclaringClassId() {
		return declaringClassId;
	}

	public void setDeclaringClassId(int declaringClassId) {
		this.declaringClassId = declaringClassId;
	}

	public int getDeclaringPackageId() {
		return declaringPackageId;
	}

	public void setDeclaringPackageId(int declaringPackageId) {
		this.declaringPackageId = declaringPackageId;
	}

	public int getMethodId() {
		return methodId;
	}

	public void setMethodId(int methodId) {
		this.methodId = methodId;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public int[][] getParamTypeIds() {
		return paramTypeIds;
	}

	public void setParamTypeIds(int[][] paramTypeIds) {
		this.paramTypeIds = paramTypeIds;
	}

	public void setParamTypes(IWorklistType[] paramTypes) {
		this.paramTypes = paramTypes;
	}

	public void setReturnType(IWorklistType returnType) {
		this.returnType = returnType;
	}

	public int getReturnPackageId() {
		return returnPackageId;
	}

	public void setReturnPackageId(int returnPackageId) {
		this.returnPackageId = returnPackageId;
	}

	public int getReturnTypeId() {
		return returnTypeId;
	}

	public void setReturnTypeId(int returnTypeId) {
		this.returnTypeId = returnTypeId;
	}

	public void setDeclaringClass(ClassInfo declaringClass) {
		this.declaringClass = declaringClass;
	}

	public Set getCallSites() {
		return callSites;
	}

	public void setCallSite(CallSiteInfo csi) {
		callSites.add(csi);
	}

	public Set getInitializedClasses() {
		if (initializedClasses == null) {
			populateInitializedClasses();
		}
		return initializedClasses;
	}

	public Set getInstantiatedClasses() {
		if (instantiatedClasses == null) {
			populateInstantiatedClasses();
		}
		return instantiatedClasses;
	}

	public void setInitializedClassId(Integer classPkgId, Integer classId) {
		initializedClassIds.add(classPkgId);
		initializedClassIds.add(classId);
	}

	public void setInstantiatedClassId(Integer classPkgId, Integer classId) {
		instantiatedClassIds.add(classPkgId);
		instantiatedClassIds.add(classId);
	}

	//	public boolean isAbstract() {
	//		//return isAbstract;
	//	}
	//

	public void setModifiers(int modifiers) {
		this.modifiers = modifiers;
	}

	// overriding methods from class Object
	public String toString() {
		String result = "Method id:" + methodId + ", " + declaringPackageId
				+ "." + declaringClassId + " " + returnPackageId + "."
				+ returnTypeId + " " + name + "(";
		for (int i = 0; i < paramTypeIds.length; i++) {
			result += (i > 0 ? "," : "") + paramTypeIds[i][0] + "."
					+ paramTypeIds[i][1];
		}
		return result + ")";
	}

	public boolean equals(Object other) {
		if (!(other instanceof MethodInfo)) {
			return false;
		}
		MethodInfo mi = (MethodInfo) other;
		if (methodId == mi.methodId) {
			return true;
		}
		return false;
	}

	public int hashCode() {
		return methodId;
	}

}
